@@ -1,8 +1,9 @@
 <?php
+
 /*
  * SPDX-License-Identifier: AGPL-3.0-only
  * SPDX-FileCopyrightText: Copyright 2005-2016 Zarafa Deutschland GmbH
- * SPDX-FileCopyrightText: Copyright 2020-2024 grommunio GmbH
+ * SPDX-FileCopyrightText: Copyright 2020-2025 grommunio GmbH
  */
 
 /**
@@ -48,7 +49,7 @@ class Recurrence extends BaseRecurrence {
 	 * @param mixed    $message  the MAPI (appointment) message
 	 * @param array    $proptags an associative array of protags and their values
 	 */
-	public function __construct($store, $message, $proptags = []) {
+	public function __construct(mixed $store, mixed $message, array $proptags = []) {
 		if (!empty($proptags)) {
 			$this->proptags = $proptags;
 		}
@@ -92,6 +93,7 @@ class Recurrence extends BaseRecurrence {
 			$properties["flagdueby"] = "PT_SYSTIME:PSETID_Common:" . PidLidReminderSignalTime;
 			$properties["side_effects"] = "PT_LONG:PSETID_Common:0x8510";
 			$properties["hideattachments"] = "PT_BOOLEAN:PSETID_Common:" . PidLidSmartNoAttach;
+			$properties['meetingrecurring'] = "PT_BOOLEAN:PSETID_Meeting:" . PidLidIsRecurring;
 
 			$this->proptags = getPropIdsFromStrings($store, $properties);
 		}
@@ -121,10 +123,18 @@ class Recurrence extends BaseRecurrence {
 			if (isset($exception_props[$this->proptags["startdate"]]) && !$this->isValidExceptionDate($base_date, $this->fromGMT($this->tz, $exception_props[$this->proptags["startdate"]]))) {
 				return false;
 			}
+			$changed_item = [];
 			// Properties in the attachment are the properties of the base object, plus $exception_props plus the base date
 			foreach (["subject", "location", "label", "reminder", "reminder_minutes", "alldayevent", "busystatus"] as $propname) {
 				if (isset($this->messageprops[$this->proptags[$propname]])) {
 					$props[$this->proptags[$propname]] = $this->messageprops[$this->proptags[$propname]];
+					if (isset($exception_props[$this->proptags[$propname]]) &&
+						$this->messageprops[$this->proptags[$propname]] != $exception_props[$this->proptags[$propname]]) {
+						$changed_item[$propname] = $exception_props[$this->proptags[$propname]];
+					}
+				}
+				elseif (isset($exception_props[$this->proptags[$propname]])) {
+					$changed_item[$propname] = $exception_props[$this->proptags[$propname]];
 				}
 			}
 
@@ -156,46 +166,16 @@ class Recurrence extends BaseRecurrence {
 			// Save the data into an attachment
 			$this->createExceptionAttachment($props, $exception_recips, $copy_attach_from);
 
-			$changed_item = [];
-
-			$changed_item["basedate"] = $basetime;
+			$changed_item["basedate"] = $baseday;
 			$changed_item["start"] = $this->fromGMT($this->tz, $props[$this->proptags["startdate"]]);
 			$changed_item["end"] = $this->fromGMT($this->tz, $props[$this->proptags["duedate"]]);
 
-			if (array_key_exists($this->proptags["subject"], $exception_props)) {
-				$changed_item["subject"] = $exception_props[$this->proptags["subject"]];
-			}
-
-			if (array_key_exists($this->proptags["location"], $exception_props)) {
-				$changed_item["location"] = $exception_props[$this->proptags["location"]];
-			}
-
-			if (array_key_exists($this->proptags["label"], $exception_props)) {
-				$changed_item["label"] = $exception_props[$this->proptags["label"]];
-			}
-
-			if (array_key_exists($this->proptags["reminder"], $exception_props)) {
-				$changed_item["reminder_set"] = $exception_props[$this->proptags["reminder"]];
-			}
-
-			if (array_key_exists($this->proptags["reminder_minutes"], $exception_props)) {
-				$changed_item["remind_before"] = $exception_props[$this->proptags["reminder_minutes"]];
-			}
-
-			if (array_key_exists($this->proptags["alldayevent"], $exception_props)) {
-				$changed_item["alldayevent"] = $exception_props[$this->proptags["alldayevent"]];
-			}
-
-			if (array_key_exists($this->proptags["busystatus"], $exception_props)) {
-				$changed_item["busystatus"] = $exception_props[$this->proptags["busystatus"]];
-			}
-
 			// Add the changed occurrence to the list
-			array_push($this->recur["changed_occurrences"], $changed_item);
+			$this->recur["changed_occurrences"][] = $changed_item;
 		}
 		else {
 			// Delete the occurrence by placing it in the deleted occurrences list
-			array_push($this->recur["deleted_occurrences"], $baseday);
+			$this->recur["deleted_occurrences"][] = $baseday;
 		}
 
 		// Turn on hideattachments, because the attachments in this item are the exceptions
@@ -227,6 +207,7 @@ class Recurrence extends BaseRecurrence {
 		for ($i = 0, $len = count($this->recur["changed_occurrences"]); $i < $len; ++$i) {
 			if ($this->isSameDay($this->recur["changed_occurrences"][$i]["basedate"], $baseday)) {
 				$extomodify = &$this->recur["changed_occurrences"][$i];
+				break;
 			}
 		}
 
@@ -238,40 +219,26 @@ class Recurrence extends BaseRecurrence {
 		// client will send basedate with time part as zero, so discard that value
 		unset($exception_props[$this->proptags["basedate"]]);
 
-		if (array_key_exists($this->proptags["startdate"], $exception_props)) {
-			$extomodify["start"] = $this->fromGMT($this->tz, $exception_props[$this->proptags["startdate"]]);
-		}
+		// Map of property keys to their target keys in $extomodify, with optional transformation
+		$propertyMappings = [
+			"startdate" => ["target" => "start", "transform" => true],
+			"duedate" => ["target" => "end", "transform" => true],
+			"subject" => ["target" => "subject", "transform" => false],
+			"location" => ["target" => "location", "transform" => false],
+			"label" => ["target" => "label", "transform" => false],
+			"reminder" => ["target" => "reminder_set", "transform" => false],
+			"reminder_minutes" => ["target" => "remind_before", "transform" => false],
+			"alldayevent" => ["target" => "alldayevent", "transform" => false],
+			"busystatus" => ["target" => "busystatus", "transform" => false],
+		];
 
-		if (array_key_exists($this->proptags["duedate"], $exception_props)) {
-			$extomodify["end"] = $this->fromGMT($this->tz, $exception_props[$this->proptags["duedate"]]);
-		}
-
-		if (array_key_exists($this->proptags["subject"], $exception_props)) {
-			$extomodify["subject"] = $exception_props[$this->proptags["subject"]];
-		}
-
-		if (array_key_exists($this->proptags["location"], $exception_props)) {
-			$extomodify["location"] = $exception_props[$this->proptags["location"]];
-		}
-
-		if (array_key_exists($this->proptags["label"], $exception_props)) {
-			$extomodify["label"] = $exception_props[$this->proptags["label"]];
-		}
-
-		if (array_key_exists($this->proptags["reminder"], $exception_props)) {
-			$extomodify["reminder_set"] = $exception_props[$this->proptags["reminder"]];
-		}
-
-		if (array_key_exists($this->proptags["reminder_minutes"], $exception_props)) {
-			$extomodify["remind_before"] = $exception_props[$this->proptags["reminder_minutes"]];
-		}
-
-		if (array_key_exists($this->proptags["alldayevent"], $exception_props)) {
-			$extomodify["alldayevent"] = $exception_props[$this->proptags["alldayevent"]];
-		}
-
-		if (array_key_exists($this->proptags["busystatus"], $exception_props)) {
-			$extomodify["busystatus"] = $exception_props[$this->proptags["busystatus"]];
+		foreach ($propertyMappings as $propKey => $mapping) {
+			$propTag = $this->proptags[$propKey];
+			if (array_key_exists($propTag, $exception_props)) {
+				$extomodify[$mapping["target"]] = $mapping["transform"] ?
+					$this->fromGMT($this->tz, $exception_props[$propTag]) :
+					$exception_props[$propTag];
+			}
 		}
 
 		$exception_props[PR_MESSAGE_CLASS] = "IPM.OLE.CLASS.{00061055-0000-0000-C000-000000000046}";
@@ -367,9 +334,9 @@ class Recurrence extends BaseRecurrence {
 	 * - reminder time of this item is not before the starttime of the previous recurring item
 	 * - reminder time of the next item is not before the starttime of this item
 	 *
-	 * @param date   $basedate        the base date of the exception (LOCAL time of non-exception occurrence)
-	 * @param string $reminderminutes reminder minutes which is set of the item
-	 * @param date   $startdate       the startdate of the selected item
+	 * @param int $basedate        the base date of the exception (LOCAL time of non-exception occurrence)
+	 * @param int $reminderminutes reminder minutes which is set of the item
+	 * @param int $startdate       the startdate of the selected item
 	 *
 	 * @returns boolean if the reminder minutes value valid (FALSE if either of the rules above are FALSE)
 	 */
@@ -406,7 +373,7 @@ class Recurrence extends BaseRecurrence {
 		return true;
 	}
 
-	public function setRecurrence($tz, $recur): void {
+	public function setRecurrence(mixed $tz, mixed $recur): void {
 		// only reset timezone if specified
 		if ($tz) {
 			$this->tz = $tz;
@@ -434,13 +401,13 @@ class Recurrence extends BaseRecurrence {
 
 	// Returns the start or end time of the occurrence on the given base date.
 	// This assumes that the basedate you supply is in LOCAL time
-	public function getOccurrenceStart($basedate) {
+	public function getOccurrenceStart(int $basedate): int {
 		$daystart = $this->dayStartOf($basedate);
 
 		return $this->toGMT($this->tz, $daystart + $this->recur["startocc"] * 60);
 	}
 
-	public function getOccurrenceEnd($basedate) {
+	public function getOccurrenceEnd(int $basedate): int {
 		$daystart = $this->dayStartOf($basedate);
 
 		return $this->toGMT($this->tz, $daystart + $this->recur["endocc"] * 60);
@@ -453,10 +420,8 @@ class Recurrence extends BaseRecurrence {
 	 * Note: Before saving this new reminder time (when snoozing), you must check for
 	 *       yourself if this reminder time is earlier than your snooze time, else
 	 *       use your snooze time and not this reminder time.
-	 *
-	 * @param mixed $timestamp
 	 */
-	public function getNextReminderTime($timestamp) {
+	public function getNextReminderTime(int $timestamp): false|int {
 		/**
 		 * Get next item from now until forever, but max 1 item with reminder set
 		 * Note 0x7ff00000 instead of 0x7fffffff because of possible overflow failures when converting to GMT....
@@ -503,7 +468,7 @@ class Recurrence extends BaseRecurrence {
 	 *
 	 * @psalm-param list{0: mixed, 1: mixed, 2?: mixed} $propsrequested
 	 */
-	public static function getCalendarItems($store, $calendar, $viewstart, $viewend, $propsrequested) {
+	public static function getCalendarItems(mixed $store, mixed $calendar, int $viewstart, int $viewend, array $propsrequested): array {
 		return getCalendarItems($store, $calendar, $viewstart, $viewend, $propsrequested);
 	}
 
@@ -519,18 +484,18 @@ class Recurrence extends BaseRecurrence {
 	public function getI18RecTypeDaily(mixed $type, mixed $interval, bool $occSingleDayRank): array {
 		switch ($interval) {
 			case 1: // workdays
-				$type = dgettext('zarafa', 'workday');
+				$type = _('workday');
 				$occSingleDayRank = true;
 				break;
 
 			case 1440: // daily
-				$type = dgettext('zarafa', 'day');
+				$type = _('day');
 				$occSingleDayRank = true;
 				break;
 
 			default: // every $interval days
 				$interval /= 1440;
-				$type = dgettext('zarafa', 'days');
+				$type = _('days');
 				$occSingleDayRank = false;
 				break;
 		}
@@ -548,24 +513,24 @@ class Recurrence extends BaseRecurrence {
 	 */
 	public function getI18RecTypeWeekly(mixed $type, mixed $interval, bool $occSingleDayRank): array {
 		if ($interval == 1) {
-			$type = dgettext('zarafa', 'week');
+			$type = _('week');
 			$occSingleDayRank = true;
 		}
 		else {
-			$type = dgettext('zarafa', 'weeks');
+			$type = _('weeks');
 			$occSingleDayRank = false;
 		}
 		$daysOfWeek = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
-		$type .= sprintf(" %s ", dgettext('zarafa', 'on'));
+		$type .= sprintf(" %s ", _('on'));
 
 		for ($j = 0, $weekdays = (int) $this->recur["weekdays"]; $j < 7; ++$j) {
 			if ($weekdays & (1 << $j)) {
-				$type .= sprintf("%s, ", dgettext('zarafa', $daysOfWeek[$j]));
+				$type .= sprintf("%s, ", _($daysOfWeek[$j]));
 			}
 		}
 		$type = trim($type, ", ");
 		if (($pos = strrpos($type, ",")) !== false) {
-			$type = substr_replace($type, " " . dgettext('zarafa', 'and'), $pos, 1);
+			$type = substr_replace($type, " " . _('and'), $pos, 1);
 		}
 
 		return [
@@ -581,11 +546,11 @@ class Recurrence extends BaseRecurrence {
 	 */
 	public function getI18RecTypeMonthly(mixed $type, mixed $interval, bool $occSingleDayRank): array {
 		if ($interval == 1) {
-			$type = dgettext('zarafa', 'month');
+			$type = _('month');
 			$occSingleDayRank = true;
 		}
 		else {
-			$type = dgettext('zarafa', 'months');
+			$type = _('months');
 			$occSingleDayRank = false;
 		}
 
@@ -603,12 +568,12 @@ class Recurrence extends BaseRecurrence {
 	public function getI18RecTypeYearly(mixed $type, mixed $interval, bool $occSingleDayRank): array {
 		if ($interval <= 12) {
 			$interval = 1;
-			$type = dgettext('zarafa', 'year');
+			$type = _('year');
 			$occSingleDayRank = true;
 		}
 		else {
 			$interval = $interval / 12;
-			$type = dgettext('zarafa', 'years');
+			$type = _('years');
 			$occSingleDayRank = false;
 		}
 
@@ -628,25 +593,21 @@ class Recurrence extends BaseRecurrence {
 		$interval = $this->recur['everyn'];
 		$occSingleDayRank = false;
 
-		switch ($type) {
-			case 0x0A: // Daily
-				return $this->getI18RecTypeDaily($type, $interval, $occSingleDayRank);
-
-			case 0x0B: // Weekly
-				return $this->getI18RecTypeWeekly($type, $interval, $occSingleDayRank);
-
-			case 0x0C: // Monthly
-				return $this->getI18RecTypeMonthly($type, $interval, $occSingleDayRank);
-
-			case 0x0D: // Yearly
-				return $this->getI18RecTypeYearly($type, $interval, $occSingleDayRank);
-		}
-
-		return [
-			'type' => $type,
-			'interval' => $interval,
-			'occSingleDayRank' => boolval($occSingleDayRank),
-		];
+		return match ($type) {
+			// Daily
+			0x0A => $this->getI18RecTypeDaily($type, $interval, $occSingleDayRank),
+			// Weekly
+			0x0B => $this->getI18RecTypeWeekly($type, $interval, $occSingleDayRank),
+			// Monthly
+			0x0C => $this->getI18RecTypeMonthly($type, $interval, $occSingleDayRank),
+			// Yearly
+			0x0D => $this->getI18RecTypeYearly($type, $interval, $occSingleDayRank),
+			default => [
+				'type' => $type,
+				'interval' => $interval,
+				'occSingleDayRank' => boolval($occSingleDayRank),
+			],
+		};
 	}
 
 	/**
@@ -666,7 +627,7 @@ class Recurrence extends BaseRecurrence {
 	 * Returns langified occurrence time.
 	 */
 	public function getI18nTime(string $format, mixed $occTime): string {
-		return gmdate(dgettext('zarafa', $format), $occTime);
+		return gmdate(_($format), $occTime);
 	}
 
 	/**
@@ -686,7 +647,7 @@ class Recurrence extends BaseRecurrence {
 			(
 				$occSingleDayRank ?
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s effective %s until %s from %s to %s.'),
+						_('Occurs every %s effective %s until %s from %s to %s.'),
 						$type,
 						$start,
 						$end,
@@ -694,7 +655,7 @@ class Recurrence extends BaseRecurrence {
 						$endocc
 					) :
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s %s effective %s until %s from %s to %s.'),
+						_('Occurs every %s %s effective %s until %s from %s to %s.'),
 						$interval,
 						$type,
 						$start,
@@ -706,13 +667,13 @@ class Recurrence extends BaseRecurrence {
 			(
 				$occSingleDayRank ?
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s effective %s until %s.'),
+						_('Occurs every %s effective %s until %s.'),
 						$type,
 						$start,
 						$end
 					) :
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s %s effective %s until %s.'),
+						_('Occurs every %s %s effective %s until %s.'),
 						$interval,
 						$type,
 						$start,
@@ -810,14 +771,14 @@ class Recurrence extends BaseRecurrence {
 			(
 				$occSingleDayRank ?
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s effective %s from %s to %s.'),
+						_('Occurs every %s effective %s from %s to %s.'),
 						$type,
 						$start,
 						$startocc,
 						$endocc
 					) :
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s %s effective %s from %s to %s.'),
+						_('Occurs every %s %s effective %s from %s to %s.'),
 						$interval,
 						$type,
 						$start,
@@ -828,12 +789,12 @@ class Recurrence extends BaseRecurrence {
 			(
 				$occSingleDayRank ?
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s effective %s.'),
+						_('Occurs every %s effective %s.'),
 						$type,
 						$start
 					) :
 					sprintf(
-						dgettext('zarafa', 'Occurs every %s %s effective %s.'),
+						_('Occurs every %s %s effective %s.'),
 						$interval,
 						$type,
 						$start
@@ -853,7 +814,7 @@ class Recurrence extends BaseRecurrence {
 		[
 			'type' => $type,
 			'interval' => $interval,
-			'occSingleDayRank' => $occSingleDayRank
+			'occSingleDayRank' => $occSingleDayRank,
 		] = $this->getI18nRecurrenceType();
 
 		// get timings of the first occurrence
@@ -959,9 +920,9 @@ class Recurrence extends BaseRecurrence {
 	/**
 	 * Function which saves the exception data in an attachment.
 	 *
-	 * @param array        $exception_props  the exception data (like any other MAPI appointment)
-	 * @param array        $exception_recips list of recipients
-	 * @param mapi_message $copy_attach_from mapi message from which attachments should be copied
+	 * @param array          $exception_props  the exception data (like any other MAPI appointment)
+	 * @param array          $exception_recips list of recipients
+	 * @param false|resource $copy_attach_from mapi message from which attachments should be copied
 	 */
 	public function createExceptionAttachment($exception_props, $exception_recips = [], $copy_attach_from = false): void {
 		// Create new attachment.
@@ -1000,7 +961,7 @@ class Recurrence extends BaseRecurrence {
 		// to change an existing exception.
 		// remove mv properties when needed
 		foreach ($props as $propTag => $propVal) {
-			if ((mapi_prop_type($propTag) & MV_FLAG) == MV_FLAG && is_null($propVal)) {
+			if ((mapi_prop_type($propTag) & MV_FLAG) == MV_FLAG && $propVal === null) {
 				unset($props[$propTag]);
 			}
 		}
@@ -1023,14 +984,7 @@ class Recurrence extends BaseRecurrence {
 	public function deleteExceptionAttachment($base_date): void {
 		$attachments = mapi_message_getattachmenttable($this->message);
 		// Retrieve only exceptions which are stored as embedded messages
-		$attach_res = [
-			RES_PROPERTY,
-			[
-				RELOP => RELOP_EQ,
-				ULPROPTAG => PR_ATTACH_METHOD,
-				VALUE => [PR_ATTACH_METHOD => ATTACH_EMBEDDED_MSG],
-			],
-		];
+		$attach_res = $this->getEmbeddedMessageRestriction();
 		$attachRows = mapi_table_queryallrows($attachments, [PR_ATTACH_NUM], $attach_res);
 
 		foreach ($attachRows as $attachRow) {
@@ -1061,19 +1015,10 @@ class Recurrence extends BaseRecurrence {
 
 	/**
 	 * Get an exception attachment based on its basedate.
-	 *
-	 * @param mixed $base_date
 	 */
-	public function getExceptionAttachment($base_date) {
+	public function getExceptionAttachment(int $base_date): mixed {
 		// Retrieve only exceptions which are stored as embedded messages
-		$attach_res = [
-			RES_PROPERTY,
-			[
-				RELOP => RELOP_EQ,
-				ULPROPTAG => PR_ATTACH_METHOD,
-				VALUE => [PR_ATTACH_METHOD => ATTACH_EMBEDDED_MSG],
-			],
-		];
+		$attach_res = $this->getEmbeddedMessageRestriction();
 		$attachments = mapi_message_getattachmenttable($this->message);
 		$attachRows = mapi_table_queryallrows($attachments, [PR_ATTACH_NUM], $attach_res);
 
@@ -1112,10 +1057,8 @@ class Recurrence extends BaseRecurrence {
 	 * @param int   $endocc       end of occurrence since beginning of day in minutes
 	 * @param mixed $tz           the timezone info for this occurrence ( applied to $basedate / $startocc / $endocc )
 	 * @param bool  $reminderonly If TRUE, only add the item if the reminder is set
-	 *
-	 * @return null|false
 	 */
-	public function processOccurrenceItem(&$items, $start, $end, $basedate, $startocc, $endocc, $tz, $reminderonly) {
+	public function processOccurrenceItem(array &$items, mixed $start, int $end, mixed $basedate, mixed $startocc, mixed $endocc, mixed $tz, mixed $reminderonly): ?false {
 		$exception = $this->isException($basedate);
 		if ($exception) {
 			return false;
@@ -1136,7 +1079,7 @@ class Recurrence extends BaseRecurrence {
 		 * the occurrence and send it in response.
 		 */
 		if (($occstart >= $end || $occend <= $start) && !($occstart == $occend && $occstart == $start)) {
-			return;
+			return null;
 		}
 
 		// Properties for this occurrence are the same as the main object,
@@ -1149,48 +1092,21 @@ class Recurrence extends BaseRecurrence {
 		$newitem["basedate"] = $basedate;
 
 		// If reminderonly is set, only add reminders
-		if ($reminderonly && (!isset($newitem[$this->proptags["reminder"]]) || $newitem[$this->proptags["reminder"]] == false)) {
-			return;
+		if ($reminderonly && (!isset($newitem[$this->proptags["reminder"]]) || $newitem[$this->proptags["reminder"]] === false)) {
+			return null;
 		}
 
 		$items[] = $newitem;
-	}
 
-	/**
-	 * processExceptionItem, adds an all exception item to a list of occurrences, without any constraint on timeframe.
-	 *
-	 * @param array $items reference to the array to be added to
-	 * @param date  $start start of timeframe in GMT TIME
-	 * @param date  $end   end of timeframe in GMT TIME
-	 */
-	public function processExceptionItems(&$items, $start, $end): void {
-		$limit = 0;
-		foreach ($this->recur["changed_occurrences"] as $exception) {
-			// Convert to GMT
-			$occstart = $this->toGMT($this->tz, $exception["start"]);
-			$occend = $this->toGMT($this->tz, $exception["end"]);
-
-			// Check range criterium. Exact matches (eg when $occstart == $end), do NOT match since you cannot
-			// see any part of the appointment. Partial overlaps DO match.
-			if ($occstart >= $end || $occend <= $start) {
-				continue;
-			}
-
-			array_push($items, $this->getExceptionProperties($exception));
-			if (count($items) == $limit) {
-				break;
-			}
-		}
+		return null;
 	}
 
 	/**
 	 * Function which verifies if on the given date an exception, delete or change, occurs.
 	 *
-	 * @param mixed $basedate
-	 *
 	 * @return bool true - if an exception occurs on the given date, false - no exception occurs on the given date
 	 */
-	public function isException($basedate) {
+	public function isException(int $basedate): bool {
 		if ($this->isDeleteException($basedate)) {
 			return true;
 		}
@@ -1204,10 +1120,8 @@ class Recurrence extends BaseRecurrence {
 
 	/**
 	 * Returns TRUE if there is a DELETE exception on the given base date.
-	 *
-	 * @param mixed $basedate
 	 */
-	public function isDeleteException($basedate): bool {
+	public function isDeleteException(int $basedate): bool {
 		// Check if the occurrence is deleted on the specified date
 		foreach ($this->recur["deleted_occurrences"] as $deleted) {
 			if ($this->isSameDay($deleted, $basedate)) {
@@ -1220,10 +1134,8 @@ class Recurrence extends BaseRecurrence {
 
 	/**
 	 * Returns the exception if there is a CHANGE exception on the given base date, or FALSE otherwise.
-	 *
-	 * @param mixed $basedate
 	 */
-	public function getChangeException($basedate) {
+	public function getChangeException(int $basedate): array|false {
 		// Check if the occurrence is modified on the specified date
 		foreach ($this->recur["changed_occurrences"] as $changed) {
 			if ($this->isSameDay($changed["basedate"], $basedate)) {
@@ -1237,12 +1149,9 @@ class Recurrence extends BaseRecurrence {
 	/**
 	 * Function to see if two dates are on the same day.
 	 *
-	 * @param mixed $date1
-	 * @param mixed $date2
-	 *
 	 * @return bool Returns TRUE when both dates are on the same day
 	 */
-	public function isSameDay($date1, $date2) {
+	public function isSameDay(int $date1, int $date2): bool {
 		$time1 = $this->gmtime($date1);
 		$time2 = $this->gmtime($date2);
 
@@ -1264,7 +1173,7 @@ class Recurrence extends BaseRecurrence {
 	 *                                   message to the attachment by default. False if only the $exception_recips changes should
 	 *                                   be applied.
 	 */
-	public function setExceptionRecipients($message, $exception_recips, $copy_orig_recips = true): void {
+	public function setExceptionRecipients(mixed $message, array $exception_recips, bool $copy_orig_recips = true): void {
 		if (isset($exception_recips['add']) || isset($exception_recips['remove']) || isset($exception_recips['modify'])) {
 			$this->setDeltaExceptionRecipients($message, $exception_recips, $copy_orig_recips);
 		}
@@ -1281,13 +1190,12 @@ class Recurrence extends BaseRecurrence {
 	 *  - "remove": This contains an array of recipients which must be removed
 	 *  - "modify": This contains an array of recipients which must be modified
 	 *
-	 * @param mixed $exception
 	 * @param array $exception_recips list of recipients
 	 * @param bool  $copy_orig_recips True to copy all recipients which are on the original
 	 *                                message to the attachment by default. False if only the $exception_recips changes should
 	 *                                be applied.
 	 */
-	public function setDeltaExceptionRecipients($exception, $exception_recips, $copy_orig_recips): void {
+	public function setDeltaExceptionRecipients(mixed $exception, array $exception_recips, bool $copy_orig_recips): void {
 		// Check if the recipients from the original message should be copied,
 		// if so, open the recipient table of the parent message and apply all
 		// rows on the target recipient.
@@ -1339,7 +1247,7 @@ class Recurrence extends BaseRecurrence {
 	 * @param resource $message          exception attachment of recurring item
 	 * @param array    $exception_recips list of recipients
 	 */
-	public function setAllExceptionRecipients($message, $exception_recips): void {
+	public function setAllExceptionRecipients(mixed $message, array $exception_recips): void {
 		$deletedRecipients = [];
 		$useMessageRecipients = false;
 
@@ -1421,7 +1329,7 @@ class Recurrence extends BaseRecurrence {
 	 *
 	 * @psalm-return false|list<mixed>
 	 */
-	public function getAllExceptions() {
+	public function getAllExceptions(): array|false {
 		if (!empty($this->recur["changed_occurrences"])) {
 			$result = [];
 			foreach ($this->recur["changed_occurrences"] as $exception) {
@@ -1442,7 +1350,7 @@ class Recurrence extends BaseRecurrence {
 	 * @param array $recipients   recipients list of message
 	 * @param bool  $isException  true if we are processing recipient of exception
 	 */
-	public function addOrganizer($messageProps, &$recipients, $isException = false): void {
+	public function addOrganizer(array $messageProps, array &$recipients, bool $isException = false): void {
 		$hasOrganizer = false;
 		// Check if meeting already has an organizer.
 		foreach ($recipients as $key => $recipient) {
@@ -1472,6 +1380,22 @@ class Recurrence extends BaseRecurrence {
 			array_unshift($recipients, $organizer);
 		}
 	}
+
+	/**
+	 * Returns restriction array for filtering embedded message attachments.
+	 *
+	 * @return array restriction array for ATTACH_EMBEDDED_MSG
+	 */
+	private function getEmbeddedMessageRestriction(): array {
+		return [
+			RES_PROPERTY,
+			[
+				RELOP => RELOP_EQ,
+				ULPROPTAG => PR_ATTACH_METHOD,
+				VALUE => [PR_ATTACH_METHOD => ATTACH_EMBEDDED_MSG],
+			],
+		];
+	}
 }
 
 /*
